#!/usr/local/bin/ruby
# $Author: mas $
# $Id: ohp.rb,v 1.5 1999/02/21 07:33:18 mas Exp $
# Copyright (C) 1998-1999 Masatoshi SEKI 

require 'quilt.rb'

class String
=begin
HTMLǤΤޤ޻Ȥʤ褦ʸִ
äȸΨɤ񤱤Ϥ
=end
  def escape_html
    s = self.dup
    s.gsub!('&', '&amp;')	# &
    s.gsub!('"', '&quot;')	# "
    s.gsub!('>', '&gt;')	# >
    s.gsub!('<', '&lt;')	# <
    s
  end

=begin
URLˤΤޤ޻Ȥʤʸ򥨥󥳡ɤ
=end
  def encode_url
    s = ''
    self.each_byte do |c|
      cs = sprintf('%c',c)
      if cs =~ /[^a-zA-Z0-9_\-.]/
	s << sprintf("%%%02X", c)
      else
	s << cs
      end
    end
    s
  end

  def decode_url
    s = self.tr('+', ' ')
    s.gsub!('%([0-9a-fA-F]{2})') { |c|
      [$1.hex].pack("c")
    }
    s
  end
end

=begin
OHPBookϥȤνޤ
=end
class OHPBook
  def initialize(filename, author, email)
    @dict = {}
    @sheets = []
    loader = OHPLoader.new(filename)
    loader.tree.each do |page|
      sheet = OHPSheet.new(page, self, author, email)
      @sheets.push(sheet)
      @dict[sheet.title] = sheet
      @dict[sheet.number] = sheet
    end
  end

  def write_html(quilt, dir=nil)
    if dir
      Dir.mkdir(dir)
      Dir.chdir(dir)
    end
    @sheets.each do |sheet|
      begin
	fp = IO.popen('coco "*sjis*dos" > ' + sheet.name, 'w')
	fp.write(sheet.build_html(quilt))
      ensure
	fp.close if fp
      end
    end
  end

=begin
sheetΥȥޤϡֹ椫sheet롣äȥХͶġ
=end
  def [] (key)
    @dict.fetch(key, nil)
  end
end

class OHPSheet
  @count = 0
  def newname
    @count = @count.succ
    [@count, "page#{@count}.html"]
  end
  module_function :newname

  def initialize(item, book, author, email)
    @author = author
    @email = email
    @number, @name = OHPSheet.newname
    @title = item.title
    @book = book
    @contents = item.contents
  end
  attr :name
  attr :number
  attr :title

  def build_ul
    ul_one(@contents)
  end

  def ul_one(items)
    return '' if (items.nil?) or (items.size <= 0)
    list = "<UL>\n"
    items.each do |item|
      if link = @book[item.title]
	li = "<LI>#{link.anchor}\n"
      else
	li = "<LI>#{item.title.escape_html}\n"
      end
      list = list + li + ul_one(item.contents)
    end
    list += "</UL>\n"
  end

  def build_html(quilt)
    quilt.piece do |e| self.sew(e) end
  end

  def sew(entity)
    case entity
    when 'AUTHOR'
      @author
    when 'EMAIL'
      @email
    when 'TITLE'
      title.escape_html
    when 'CONTENTS'
      build_ul
    when 'LINK_NEXT'
      @book[number+1] ? @book[number+1].anchor : ''
    when 'LINK_PREV'
      @book[number-1] ? @book[number-1].anchor : ''
    when 'LINK_INDEX'
      @book[1] ? @book[1].anchor : ''
    else
      nil
    end
  end

  def anchor
    %Q!<A href="#{self.name}">#{self.title.escape_html}</A>!
  end
end

class OHPItem
  def initialize(str)
    @title = str
    @contents = nil
  end
  attr :title
  attr :contents

  def push(item)
    if @contents
      @contents.push(item)
    else
      @contents = [item]
    end
  end

  def extend_string(str)
    if @contents
      @contents[-1] = @contents[-1]+str
    else
      push(str)
    end
  end

  def each
    if @contents
      @contents.each do |item|
	yield item
      end
    end
  end
end

=begin
վեɤǡOHPItemڤ
=end
class OHPLoader
  def initialize(fname)
    @tree = OHPItem.new(fname)
    @curr = @tree
    @sym_stack = []
    @tree_stack = [@tree]
    load_file(fname)
  end
  attr :tree

  def load_file(fname)
    begin 
      fp = IO.popen('coco "*euc-japan*unix" <' + fname)
      while (ln = fp.gets)
	ln.chomp!
	ln.sub!(/^\s+/, '')
	ln.sub!(/\s+$/, '')
	if ln.size > 0
	  load_one(ln)
	end
      end
    ensure
      fp.close
    end
  end

  def load_one(str)
    sym, str = split_symbol(str)
    if sym 
      if @sym_stack.include?(sym)
	@tree_stack.pop
	while sym != @sym_stack.pop
	  @tree_stack.pop
	end
	@curr = @tree_stack[-1]
      end
      @sym_stack.push sym
      item = OHPItem.new(str)
      @curr.push(item)
      @curr = item
      @tree_stack.push(item)
    else
      @curr.extend_string(str)
    end
  end

  def split_symbol(str)
    sym = nil
    if str =~ /^(.)/
      if OHPLoader.symbol?($1)
	sym = $1.dup
	str = str.sub(/^.\s*/, '')
      end
    end
    [sym, str]
  end

  def symbol?(s)
    case s
    when '', '', '', '', 'o', '', '', ''
      true
    else
      nil
    end
  end
  module_function :symbol?
end

if __FILE__ == $0
  quilt = Quilt.new_with_file(ARGV.shift)
  ohp = OHPBook.new(ARGV.shift, 'Masatoshi SEKI', 'm_seki@mva.biglobe.ne.jp')
  ohp.write_html(quilt, 'tmp')
end
